# Copyright (c) 2010 Outpost Embedded, LLC
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program; if
# not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA. Any Outpost Embedded trademarks that are
# incorporated in the source code or documentation are not subject to the GNU
# General Public License and may only be used or replicated with the express
# permission of Outpost Embedded, LLC.

from fdisk import units


class BaseDisk(object):
    def get_name(self):
        raise NotImplementedError()

    def get_manufacturer(self):
        raise NotImplementedError()

    def get_model(self):
        raise NotImplementedError()

    def get_size(self):
        raise NotImplementedError()

    def get_device_path(self):
        raise NotImplementedError()

    def is_removable(self):
        raise NotImplementedError()

    ###

    def get_ui_string(self):
        size = units.bytes(self.get_size()).normalize_approx(family = units.SI)
        size_desc = size.get_description(abbreviate = True)
        return u'%s %s (%s) [%s]' % (
          self.get_manufacturer(),
          self.get_model(),
          size_desc,
          self.get_name(),
        )

    ###

    def start(self):
        pass

    def stop(self):
        pass

    def create_partition_table(self):
        raise NotImplementedError()

    def create_filesystem(self):
        raise NotImplementedError()

    def install_bootloader(self):
        raise NotImplementedError()

    def mount(self):
        raise NotImplementedError()

    def umount(self):
        raise NotImplementedError()

    # FIXME: sync is a bit redundant given that we unmount the device.
    def sync(self):
        raise NotImplementedError()


class BaseBackend(object):
    add_disk = None
    remove_disk = None

    def __init__(self, add_disk, remove_disk):
        super(BaseBackend, self).__init__()
        self.add_disk = add_disk
        self.remove_disk = remove_disk

    def process_thread(self, thread):
        pass

    def start(self):
        pass

    def stop(self):
        pass
