# Copyright (c) 2010 Outpost Embedded, LLC
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program; if
# not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA. Any Outpost Embedded trademarks that are
# incorporated in the source code or documentation are not subject to the GNU
# General Public License and may only be used or replicated with the express
# permission of Outpost Embedded, LLC.

import os, sys, errno
from logging import (
  Logger as _Logger,
  Handler,
  FileHandler,
  StreamHandler,
  DEBUG,
  INFO,
  WARNING,
  ERROR,
)


class Logger(_Logger):
    @classmethod
    def ToFile(cls, filename, level = DEBUG):
        filename = os.path.abspath(filename)

        parent = os.path.dirname(filename)
        try:
            os.makedirs(parent)
        except OSError, e:
            if e.errno != errno.EEXIST:
                raise

        instance = cls(level = level)
        instance.addHandler(FileHandler(filename))
        return instance

    @classmethod
    def ToStream(cls, stream = None, level = DEBUG):
        instance = cls(level = level)
        if stream is None:
            stream = sys.stderr
        instance.addHandler(StreamHandler(stream))
        return instance

    @classmethod
    def ToNull(cls):
        instance = cls(level = ERROR)
        instance.addHandler(NullHandler())
        return instance


class ErrorLogger(Logger):
    def __init__(self, name = None, level = DEBUG):
        Logger.__init__(self, 'rapidwriter.error', level = level)

    def traceback(self):
        import traceback
        self.error(traceback.format_exc())


default_logger = ErrorLogger.ToStream()


def set_default_logger(logger):
    global default_logger
    default_logger = logger


def log_debug(*args):
    default_logger.debug(*args)


def log_info(*args):
    default_logger.info(*args)


def log_warning(*args):
    default_logger.warning(*args)


def log_error(*args):
    default_logger.error(*args)


def log_traceback():
    default_logger.traceback()


def make_logging_callback(s):
    def cb(*args):
        log_debug('%s (%s)' % (s, ', '.join(repr(a) for a in args)))
    return cb


def get_user_log_level(desc):
    if desc == 'debug':
        return DEBUG
    elif desc == 'info':
        return INFO
    elif desc == 'warning':
        return WARNING
    elif desc == 'error':
        return ERROR
    else:
        raise ValueError(desc)


def get_user_logger(path, level_desc):
    level = get_user_log_level(level_desc)
    if path == '-':
        logger = ErrorLogger.ToStream(level = level)
    else:
        logger = ErrorLogger.ToFile(path, level = level)
    return logger
